<?php
namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Notification;
use \Zeedhi\Framework\DTO\Request\Filter;
use \Zeedhi\Framework\DTO\Response;
use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class PlaValidacaoCardapio extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $parameterBagColumns;
    protected $environment;
    protected $estadoDoCardapio;
    protected $tipoDeVisualizacao;
    protected $dataSourceName;
    protected $statusCardapio;
    protected $compcarddel;
    protected $entityManager;
    protected $authController;
    protected $util;
    protected $libUtilities;
    protected $connection;
    protected $novoCodigo;
    
    const VALIDACAO_CARDAPIO = "pla_validacao_cardapio";

    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $util, $newCode, $entityManager, $authController,$libUtilities,$connection,$novoCodigo, $reportService) {
        $this->crudService          = $crudService;
        $this->parameterBag         = $parameterBag;
        $this->dataSourceManager    = $dataSourceManager;
        $this->environment          = $environment;
        $this->entityManager        = $entityManager;
        $this->authController       = $authController;
        $this->util                 = $util;
        $this->libUtilities         = $libUtilities;
        $this->connection           = $connection;
        $this->novoCodigo           = $novoCodigo;
        $this->reportService        = $reportService;
    }
    
    public function verificaComensaisSemPratoCardapios(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        
        $idpercomensprat = $this->verificaParamComensaisSemPrato($row);
        $cardapiosComensaisSemPrato = [];
        
        if($idpercomensprat == 'N'){
            
             foreach($row as $card){
             
                 $comensaisSemPrato = $this->retornaCardComemsaisSemPrato($card);
                 if(!empty($comensaisSemPrato)){
                     array_push($cardapiosComensaisSemPrato,$comensaisSemPrato);
                 }
                 
             }
            
        if(!empty($cardapiosComensaisSemPrato)){
            $response->addDataSet(new DataSet ('comensaisSemPrato', $cardapiosComensaisSemPrato));
        }
            
            
        }
        
    }
    
    //Valida Comensais Sem Prato
    
    public function verificaParamComensaisSemPrato($row) {

        $query = "SELECT IDPERCOMENSPRAT FROM PARAMGERAL";
        
        $idpercomensprat = $this->entityManager->getConnection()->fetchAll($query);
        return $idpercomensprat[0]['IDPERCOMENSPRAT'];
        
    }
    
    public function retornaCardComemsaisSemPrato($row){
        
        $query = "SELECT CD.CDFILIAL ||  ' - '  || F.NMFILIAL AS CDFILIAL,
                         CD.CDSERVFILI || ' - ' || S.NMSERVICO AS CDSERVFILI,
                         CD.DTCARDAPIO , CD.NRCARDAPIO
                FROM CARDADIA CD
                LEFT JOIN FILIAL F ON CD.CDFILIAL = F.CDFILIAL
                LEFT JOIN SERVICO S ON CD.CDFILIAL = S.CDFILIAL
                                    AND CD.CDSERVFILI = S.CDSERVFILI
                WHERE NOT EXISTS(SELECT * FROM COMPCARD C
                                    WHERE C.CDFILIAL = :CDFILIAL
                                        AND C.NRCARDAPIO = CD.NRCARDAPIO)
                      AND CD.CDFILIAL = :CDFILIAL
                      AND CD.CDSERVFILI = :CDSERVFILI
                      AND CD.DTCARDAPIO BETWEEN TO_DATE(:DATA_INICIAL, 'DD/MM/YYYY') AND TO_DATE(:DATA_FINAL, 'DD/MM/YYYY')";
                      
        $param = array(
            
            'CDFILIAL' => $row['CDFILIAL'],
            'CDSERVFILI' => $row['CDSERVFILI'],
            'DATA_INICIAL' => $row['DATA_INICIAL'],
            'DATA_FINAL' => $row['DATA_FINAL']
        );
            
        $comensaisSemPrato = $this->entityManager->getConnection()->fetchAll($query, $param);
        return !empty($comensaisSemPrato) ? $comensaisSemPrato : array() ;
    }
    
    public function deleteComensaisSemPrato(DTO\Request $request, DTO\Response $response){
        
        try{
            $this->connection->beginTransaction();
            $row = $request->getRow();

            foreach($row as $rows){
                
                $cdfilial = substr($rows['CDFILIAL'], 0, 4);     
                $cdservfili = substr($rows['CDSERVFILI'], 0, 5);
                $dtcardapio = $rows['DTCARDAPIO'];
                
                $sql = "DELETE FROM CARDADIA 
    			WHERE CDFILIAL = :CDFILIAL AND
       				CDSERVFILI = :CDSERVFILI  AND
       				DTCARDAPIO = :DTCARDAPIO";
    
        		$params = array(
                    'CDFILIAL' => $cdfilial,
                    'CDSERVFILI' => $cdservfili,
                    'DTCARDAPIO' => $dtcardapio
        		);
        
        		$query = $this->entityManager->getConnection()->prepare($sql);
                $query->execute($params);
                
            }

            
            $this->connection->commit();
        }catch(\Exception $e){
            $this->connection->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            
        }
        
    }
    
    //Chama todas as validações restantes para vários cardápios
    public function validacoesCardapios(DTO\Request $request, DTO\Response $response){
        //Limpeza da tabela. Apaga todo o log do início até 2 dias atrás
        $this->apagaLogValidacaoAntiga();
        $row = $request->getRow();
        
        $naoValidados = [];
        
        $nrValidacao = $this->retornaNrValidacao();
        
        foreach($row as $cards){
            
            $validacaoComensaisTipoPratoNaoOpcao = $this->validaComensaisTipoPratoNaoOpcao($cards);
        
            $validacaoCustoCardapioForaPadrao = $this->validaCustoCardapioForaPadrao($cards);
            
            $validacaoRestricoesContratuais = $this->validaRestricoesContratuais($cards);
            
            $validacoes = array(
                
                'ComensaisTipoPratoNaoOpcao' => $validacaoComensaisTipoPratoNaoOpcao,
                'CustoCardapioForaPadrao' => $validacaoCustoCardapioForaPadrao,
                'RestricoesContratuais' => $validacaoRestricoesContratuais
                
            );
                   
            
            if(empty($validacoes['ComensaisTipoPratoNaoOpcao']) && empty($validacoes['CustoCardapioForaPadrao']) && 
                    empty($validacoes['RestricoesContratuais']['FREQUENCIA_PRATO']) && empty($validacoes['RestricoesContratuais']['FREQUENCIA_GRUPO_PRATO']) &&
                    empty($validacoes['RestricoesContratuais']['FREQUENCIA_PRODUTO_GRUPO_PRATO']) && empty($validacoes['RestricoesContratuais']['RESTRICAO_PRODUTO_GRUPO_PRATO']) &&
                    empty($validacoes['RestricoesContratuais']['FREQUENCIA_GRUPO_PRATO_OPCAO'])){
                    
                $result = array(
                    'CDFILIAL' => $cards['CDFILIAL'],
                    'CDSERVFILI' => $cards['CDSERVFILI'],
                    'DTMESCARDAPIO' =>$cards['DATA_INICIAL']
                );
                
                $this->updateValidaCardapio($cards);
                
            }else{
                $result = array(
                    'CDFILIAL' => $cards['CDFILIAL'],
                    'CDSERVFILI' => $cards['CDSERVFILI'],
                    'DTMESCARDAPIO' =>$cards['DATA_INICIAL'],
                    'VALIDACOES' => $validacoes
                    
                );
                $this->insereLogValidacao($nrValidacao, $result);
                array_push($naoValidados,$result);
            }
            
            
            
        }
        $url = $this->reportService->createRemoteReport('BIRT', 'PLA12500', [
            'P_NRORG' => $this->environment->getNrOrgTrab(),
            'P_NRVALIDACAO' => $nrValidacao
        ]);
        $response->addDataSet(new DataSet ('NAO_VALIDADOS', $naoValidados));
        if(count($naoValidados) > 0){
            $response->addDataSet(new DataSet ('rel', ['url' => $url]));
        }
    }
    
    
    //Valida Comensais de Tipo de Prato não Opção
    
    public function validaComensaisTipoPratoNaoOpcao($row){
        
        $idsomacomest = $this->verificaParamComensaisTipoPratoNaoOpcao($row);

        if($idsomacomest == 'S'){
            
            $cardapioComensaisForaValorBase = $this->retornaCardComensaisForaValorBase($row);
            
            if(!empty($cardapioComensaisForaValorBase)){
              
                return array(
                    'comensaisForaValorBase' => $cardapioComensaisForaValorBase
                );
                
            }
        }
        
    }
    
    public function retornaCardComensaisForaValorBase($row){
        
        $query =    "SELECT
                       TO_CHAR(DTCARDAPIO, 'DD/MM/YYYY') AS DTCARDAPIO,
                       CDTIPOPRAT,
                       COMENSEST,
                       COMENSDIA,
                       DIFERENCA
                    FROM
                       (SELECT
                           C.DTCARDAPIO,
                           CC.CDTIPOPRAT,
                           ROUND(SUM(CC.VRPECONSCARD / 100 * C.QTCOMESTICAR),2) AS COMENSEST,
                           C.QTCOMESTICAR AS COMENSDIA,
                           ROUND(SUM(CC.VRPECONSCARD / 100 * C.QTCOMESTICAR) - C.QTCOMESTICAR, 2) AS DIFERENCA
                        FROM
                           CARDADIA C,
                           COMPCARD CC,
                           CNFCAMES CF
                        WHERE
                           C.CDFILIAL      = :P_CDFILIAL      AND
                           C.CDSERVFILI    = :P_CDSERVICO    AND
                           C.DTMESCARD     = :P_DTMES        AND
                           C.CDFILIAL      = CC.CDFILIAL     AND
                           C.NRCARDAPIO    = CC.NRCARDAPIO   AND
                           C.CDFILIAL      = CF.CDFILIAL     AND
                           C.CDSERVFILI    = CF.CDSERVFILI   AND
                           C.DTMESCARD     = CF.DTREFECNFMES AND
                           CC.NRSEQPRATCAR = CF.NRSEQCNFMES  AND
                           CC.CDTIPOPRAT   = CF.CDTIPOPRAT   AND
                           CF.IDOPCCNFMES  <> 'S'            AND
                           CF.IDCONCNFMES  = 'S'             AND
                           C.DTCARDAPIO    >= :P_DTPROC
                        GROUP BY
                           C.DTCARDAPIO,
                           C.QTCOMESTICAR,
                           CC.CDTIPOPRAT
                        HAVING
                           ROUND(SUM(CC.VRPECONSCARD / 100 * C.QTCOMESTICAR), 3) > ROUND(C.QTCOMESTICAR, 3))";
                      
        $param = array(
            
            'P_CDFILIAL' => $row['CDFILIAL'],
            'P_CDSERVICO' => $row['CDSERVFILI'],
            'P_DTMES' => $row['DATA_INICIAL'],
            'P_DTPROC' => $row['DATA_INICIAL']
        );
           
        $comensaisForaValorBase = $this->entityManager->getConnection()->fetchAll($query, $param);
        return !empty($comensaisForaValorBase) ? $comensaisForaValorBase : array() ;
        
    }
    
    public function verificaParamComensaisTipoPratoNaoOpcao($row){
        
        $query = "SELECT IDSOMACOMEST FROM PARAMPLANEJA
                    WHERE CDFILIAL = :CDFILIAL";
        
        $param = array(
            
            'CDFILIAL' => $row['CDFILIAL']
            
        );
        
        $idpercomensprat = $this->entityManager->getConnection()->fetchAll($query,$param);
        return $idpercomensprat[0]['IDSOMACOMEST'];
        
    }
    
    
    //Valida Custo do Cardápio Fora do Padrão
    
    public function validaCustoCardapioForaPadrao($row){
        
        $idblqfechcard = $this->verificaParamCustoCardapioForaPadrao($row);
        
        if($idblqfechcard == 'S'){
            
            $nrconsplan = $this->novoCodigo->getCodeByTableName('CONSPLAN'.$row['CDFILIAL'], $this->environment->getNrOrg());
            
            $row['CDOPERADOR'] = $this->environment->getCdOperador();
            $row['NRCONSPLAN'] = $nrconsplan;
            $this->executaProcedureConsumoPlanejado($row,$nrconsplan);
         
            $valoresCustoCard = $this->retornaCustoCard($row);
          
            $this->deleteConsplan($nrconsplan);
      
            if(!empty($valoresCustoCard)){
                
                if(($valoresCustoCard['DIFECUSTO'] > 0) && ($valoresCustoCard['DIFECUSTO'] > $valoresCustoCard['VRPEMAXDIFPLAPA']) ){
                  
                  return array(
                      'custoForaMeta' => $valoresCustoCard
                  );
                  
                }else if (($valoresCustoCard['DIFECUSTO'] < 0) && (abs($valoresCustoCard['DIFECUSTO']) > $valoresCustoCard['VRPEMINDIFPLAPA']) ){
                  
                  return array(
                      'custoForaMeta' => $valoresCustoCard
                  );
                  
                }
            }
        }
    }
    
    public function retornaCustoCard($row){
        
        $query ="SELECT CUS.CUSTO,PP.VRPEMAXDIFPLAPA, PP.VRPEMINDIFPLAPA, ROUND(((CUS.CUSTO * 100 / CUSTPADR.VRCUSTPADR) -100), 5) AS DIFECUSTO,
                        CUSTPADR.VRCUSTPADR * ((100 - NVL(PP.VRPEMINDIFPLAPA,0))/100) AS VRCUSTOMIN,
                        CUSTPADR.VRCUSTPADR * ((100 + NVL(PP.VRPEMINDIFPLAPA,0))/100) AS VRCUSTOMAX,
                        CUSTPADR.VRCUSTPADR,
                        NVL(CUS.CUSTO, 0) - CUSTPADR.VRCUSTPADR AS DIFERENCA
                            FROM CUSTPADR, PARAMPLANEJA PP,
                    (SELECT (CUSTO.VRCUSTO / QT.QUANTIDADE)AS CUSTO
                            FROM
                            (SELECT SUM(CUSTO_PRODUTO(CP.CDFILICPLA, CP.CDPRODCPLA, :P_CDTIPOCUSTO, :P_DTREFERENCIA , SYSDATE) * CP.QTCONSPLAN) AS VRCUSTO
                            FROM CONSPLAN CP
                            WHERE (CP.NRCONSPLAN = :P_NRCONSPLAN)
                               AND (CP.CDFILICPLA = :P_CDFILIAL)
                               AND (CP.CDSERVCPLA = :P_CDSERVFILI)) CUSTO,
                    (SELECT SUM(DECODE(PF.IDCOMCUSPERC,'C',CA.QTCLIESTICAR,CA.QTCOMESTICAR)) AS QUANTIDADE
                            FROM CARDADIA CA, PARAMFILIAL PF
                            WHERE (CA.CDFILIAL = :P_CDFILIAL)
                               AND (CA.CDSERVFILI = :P_CDSERVFILI)
                               AND (CA.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL)
                               AND (PF.CDFILIAL = CA.CDFILIAL))QT) CUS
                    WHERE PP.CDFILIAL = :P_CDFILIAL
                      AND (PP.CDFILIAL = CUSTPADR.CDFILIAL)
                      AND (CUSTPADR.DTREFCUSPADR = :P_DTINICIAL)
                      AND (CUSTPADR.CDSERVFILI = :P_CDSERVFILI)";
                      
        $param = array(
            
            'P_CDFILIAL' => $row['CDFILIAL'],
            'P_CDSERVFILI' => $row['CDSERVFILI'],
            'P_CDTIPOCUSTO'=> $row['TIPO_CUSTO'],
            'P_NRCONSPLAN' => $row['NRCONSPLAN'],
            'P_DTINICIAL' => $row['DATA_INICIAL'],
            'P_DTFINAL' => $row['DATA_FINAL'],
            'P_DTREFERENCIA' => $row['DATA_REFERENCIA']
            
        );

        $custoCardapio = $this->entityManager->getConnection()->fetchAll($query, $param);
        return !empty($custoCardapio) ? $custoCardapio[0] : array() ;
        
    }
    
    public function executaProcedureConsumoPlanejado ($row,$nrconsplan) {

        $connection = $this->entityManager->getConnection();
        $procedureConsumoPlanejado = new StoredProcedure($connection, 'CONSUMO_PLANEJADO_PLA');
        $procedureConsumoPlanejado->addParam(new Param('P_FILIAL',       Param::PARAM_INPUT, $row['CDFILIAL'],         Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_SERVICOS',     Param::PARAM_INPUT, $row['CDSERVFILI'],  Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DATA_INICIAL'],  Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DATA_FINAL'],  Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, "C",                Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, "CFP",              Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, "P",                Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, 0,                  Param::PARAM_TYPE_INT));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, "N",                Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $nrconsplan,        Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, "0",                Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, "zzzzzzzzzz",       Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $row['CDOPERADOR'], Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTINICIAL',  Param::PARAM_INPUT, "",                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTFINAL',    Param::PARAM_INPUT, "",                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->execute();

    }
    
    public function deleteConsplan($nrconsplan){
        $param = array("NRCONSPLAN" => $nrconsplan);
        $query = "DELETE FROM CONSPLAN
                    WHERE NRCONSPLAN = :NRCONSPLAN";
        $this->connection->executeQuery($query, $param);
    }
    
    public function verificaParamCustoCardapioForaPadrao($row){
        
        $query = "SELECT IDBLQFECHCARD FROM PARAMPLANEJA
                    WHERE CDFILIAL = :CDFILIAL";
        
        $param = array(
            
            'CDFILIAL' => $row['CDFILIAL']
            
        );
        
        $idpercomensprat = $this->entityManager->getConnection()->fetchAll($query,$param);
        return $idpercomensprat[0]['IDBLQFECHCARD'];
        
    }
    
    //Validações Restrições Contratuais
    
    public function validaRestricoesContratuais($row){
         
         $idrestrcard = $this->verificaParamRestricoesContratuaisCard($row);
     
         if($idrestrcard == 'S'){
        
            $resultadoFreqPratoMesSemana = $this->retornaRestricaoFreqPratoMesSemana($row);
            
            $resultadoFreqGrupoPratoMesSemana = $this->retornaRestricaoFreqGrupoPratoMesSemana($row);
      
            $resultadoFreqProdGrupoPratoMesSemana = $this->retornaRestricaoFreqProdGrupoPratoMesSemana($row);
            
            $resultadoRestProdGrupoPrato = $this->retornaRestriProdGrupoPrato($row);
            
            $resultadoFreqGrupoPratoOpcao = $this->retornaFreqGrupoPratoOpcao($row);
            
            $resultadoLimitPerCapita = $this->retornaLimitPerCapita($row);
          
            $result = array(
                
                'FREQUENCIA_PRATO' => (!empty($resultadoFreqPratoMesSemana['restricaoFreqPratoMaxSemana']) || 
                                        !empty($resultadoFreqPratoMesSemana['restricaoMensalFreqPratoMax']) || 
                                        !empty($resultadoFreqPratoMesSemana['restricaoMensalFreqPratoMin'])) ? $resultadoFreqPratoMesSemana : array() ,
                                        
                'FREQUENCIA_GRUPO_PRATO' => (!empty($resultadoFreqGrupoPratoMesSemana['restricaoFreqGrupoPratoMaxSemana']) || 
                                        !empty($resultadoFreqGrupoPratoMesSemana['restricaoMensalFreqGrupoPratoMax']) || 
                                        !empty($resultadoFreqGrupoPratoMesSemana['restricaoMensalFreqGrupoPratoMin'])) ? $resultadoFreqGrupoPratoMesSemana : array() ,
                                        
                'FREQUENCIA_PRODUTO_GRUPO_PRATO' => (!empty($resultadoFreqProdGrupoPratoMesSemana['restricaoFreqProdGrupoPratoMaxSemana']) || 
                                        !empty($resultadoFreqProdGrupoPratoMesSemana['restricaoMensalProdFreqGrupoPratoMax']) || 
                                        !empty($resultadoFreqProdGrupoPratoMesSemana['restricaoMensalProdFreqGrupoPratoMin'])) ? $resultadoFreqProdGrupoPratoMesSemana : array() ,
                                        
                'RESTRICAO_PRODUTO_GRUPO_PRATO' => (!empty($resultadoRestProdGrupoPrato['restricaoProdGrupoPrato']))  ? $resultadoRestProdGrupoPrato : array() ,
                
                'FREQUENCIA_GRUPO_PRATO_OPCAO' => (!empty($resultadoFreqGrupoPratoOpcao['restricaoFreqGrupoPratoOpcaoMaxSemana']) || 
                                        !empty($resultadoFreqGrupoPratoOpcao['restricaoMensalFreqGrupoPratoOpcaoMax']) || 
                                        !empty($resultadoFreqGrupoPratoOpcao['restricaoMensalFreqGrupoPratoOpcaoMin'])) ? $resultadoFreqGrupoPratoOpcao : array() ,
                                        
                'LIMITE_PER_CAPITA_POR_SERVICO' => (!empty($resultadoLimitPerCapita['restricaoLimitPerCapitaMax']) || 
                                        !empty($resultadoLimitPerCapita['restricaoLimitPerCapitaMin'])) ? $resultadoLimitPerCapita : array() 
                
            );
           
          
          return $result;
            
        }
    }
    
    public function retornaLimitPerCapita($row){
             $restricaoAtivaMensalMax = [];
             $restricaoAtivaMensalMin = [];
             
             $dataInicial = $row['DATA_INICIAL'];
             $dataFinal = $row['DATA_FINAL'];
             
             //Verifica restrição mensal frequência produto grupo de prato
             $dadosRestricaoMensal = $this->retornaDadosLimitPerCapita($row,$dataInicial,$dataFinal);
         
             if(!empty($dadosRestricaoMensal)){
                 foreach($dadosRestricaoMensal as $restricaoMensal){
                     
                     if($restricaoMensal['QTDE_PLAN'] > $restricaoMensal['QTMAXLIMPERSERGR']){
                         
                         array_push($restricaoAtivaMensalMax, $restricaoMensal );
                     }
                     
                     if($restricaoMensal['QTDE_PLAN'] < $restricaoMensal['QTMINLIMPERSERGR']){
                         
                         array_push($restricaoAtivaMensalMin, $restricaoMensal );
                     }
                }
                
             }
         
        $result = array(
            
            'restricaoLimitPerCapitaMax' => $restricaoAtivaMensalMax,
            'restricaoLimitPerCapitaMin' => $restricaoAtivaMensalMin
            
        );
        
        return $result;
    }
    
    public function retornaFreqGrupoPratoOpcao($row){
        
             $restricaoAtivaMensalMax = [];
             $restricaoAtivaMensalMin = [];
             $restricaoAtivaSemanal = [];
             
             $dataInicial = $row['DATA_INICIAL'];
             $dataFinal = $row['DATA_FINAL'];
             
             //Verifica restrição mensal frequência produto grupo de prato
             $dadosRestricaoMensal = $this->retornaDadosFreqGrupoPratoOpcao($row,$dataInicial,$dataFinal);
         
             if(!empty($dadosRestricaoMensal)){
                 foreach($dadosRestricaoMensal as $restricaoMensal){
                     
                     if($restricaoMensal['QTDE_GRUPO'] > $restricaoMensal['NRMAXFREQOPCAGR']){
                         
                         array_push($restricaoAtivaMensalMax, $restricaoMensal );
                     }
                     
                     if($restricaoMensal['QTDE_GRUPO'] < $restricaoMensal['NRMINFREQOPCAGR']){
                         
                         array_push($restricaoAtivaMensalMin, $restricaoMensal );
                     }
                }
                
             }
         
         
             //Verifica restrição semanal frequência produto grupo de prato
             foreach($row['PRIMEIRO_ULTIMO_DIA_SEMANA'] as $dtcard){

                $dataInicial = $dtcard['PRIMEIRO_DIA'];
                $dataFinal = $dtcard['ULTIMO_DIA'];
                $nrSemana = $dtcard['NRSEMANA'];
                 
                $dadosRestricaoSemanal = $this->retornaDadosFreqGrupoPratoOpcao($row,$dataInicial,$dataFinal,$nrSemana);
                
                if(!empty($dadosRestricaoSemanal)){
                 foreach($dadosRestricaoSemanal as $restricaoSemanal){
                    
                     if($restricaoSemanal['QTDE_GRUPO'] > $restricaoSemanal['NRMAXFREQOPCAGRSEM']){
                         
                         array_push($restricaoAtivaSemanal, $restricaoSemanal );
                     }
                }
                
            }
        }
        
        $result = array(
            
            'restricaoFreqGrupoPratoOpcaoMaxSemana' => $restricaoAtivaSemanal,
            'restricaoMensalFreqGrupoPratoOpcaoMax' => $restricaoAtivaMensalMax,
            'restricaoMensalFreqGrupoPratoOpcaoMin' => $restricaoAtivaMensalMin
            
        );
        
        return $result;
    }
    
    public function retornaRestriProdGrupoPrato($row){
        
        $dadosRestricao = $this->retornaDadosRestricaoProdGrupoPrato($row);
        $result = null;
        if($dadosRestricao){
            
            $result = array(
                'restricaoProdGrupoPrato' => $dadosRestricao
            );
        }
        
        return $result ? $result : array();
        
    }

    
    public function retornaRestricaoFreqProdGrupoPratoMesSemana($row){
             $restricaoAtivaMensalMax = [];
             $restricaoAtivaMensalMin = [];
             $restricaoAtivaSemanal = [];
             
             $dataInicial = $row['DATA_INICIAL'];
             $dataFinal = $row['DATA_FINAL'];
             
             //Verifica restrição mensal frequência produto grupo de prato
             $dadosRestricaoMensal = $this->retornaDadosRestricaoFreqProdGrupoPrato($row,$dataInicial,$dataFinal);
          
             if(!empty($dadosRestricaoMensal)){
                 foreach($dadosRestricaoMensal as $restricaoMensal){
                     
                     if($restricaoMensal['QTDE_PROD'] > $restricaoMensal['NRMAXPRODGRPRAT']){
                         
                         array_push($restricaoAtivaMensalMax, $restricaoMensal );
                     }
                     
                     if($restricaoMensal['QTDE_PROD'] < $restricaoMensal['NRMINPRODGRPRAT']){
                         
                         array_push($restricaoAtivaMensalMin, $restricaoMensal );
                     }
                }
                
             }

         
             //Verifica restrição semanal frequência produto grupo de prato
             foreach($row['PRIMEIRO_ULTIMO_DIA_SEMANA'] as $dtcard){

                $dataInicial = $dtcard['PRIMEIRO_DIA'];
                $dataFinal = $dtcard['ULTIMO_DIA'];
                $nrSemana = $dtcard['NRSEMANA'];
                 
                $dadosRestricaoSemanal = $this->retornaDadosRestricaoFreqProdGrupoPrato($row,$dataInicial,$dataFinal,$nrSemana);
                
                if(!empty($dadosRestricaoSemanal)){
                 foreach($dadosRestricaoSemanal as $restricaoSemanal){
                    
                     if($restricaoSemanal['QTDE_PROD'] > $restricaoSemanal['NRMAXPRODGRPRATSEM']){
                         
                         array_push($restricaoAtivaSemanal, $restricaoSemanal );
                     }
                }
                
            }
        }
        
        $result = array(
            
            'restricaoFreqProdGrupoPratoMaxSemana' => $restricaoAtivaSemanal,
            'restricaoMensalProdFreqGrupoPratoMax' => $restricaoAtivaMensalMax,
            'restricaoMensalProdFreqGrupoPratoMin' => $restricaoAtivaMensalMin
            
        );
        
        return $result;
    }
    
    
    public function retornaRestricaoFreqGrupoPratoMesSemana($row){
             $restricaoAtivaMensalMax = [];
             $restricaoAtivaMensalMin = [];
             $restricaoAtivaSemanal = [];
             
             $dataInicial = $row['DATA_INICIAL'];
             $dataFinal = $row['DATA_FINAL'];
             
             //Verifica restrição mensal frequência grupo de prato
             $dadosRestricaoMensal = $this->retornaDadosRestricaoFreqGrupoPrato($row,$dataInicial,$dataFinal);
            
             if(!empty($dadosRestricaoMensal)){
                 foreach($dadosRestricaoMensal as $restricaoMensal){
                     
                     if($restricaoMensal['QTDE_GRUPO'] > $restricaoMensal['NRMAXFREQGRPRATMES']){
                         
                         array_push($restricaoAtivaMensalMax, $restricaoMensal );
                     }
                     
                     if($restricaoMensal['QTDE_GRUPO'] < $restricaoMensal['NRMINFREQGRPRATMES']){
                         
                         array_push($restricaoAtivaMensalMin, $restricaoMensal );
                     }
                }
                
             }
         
             //Verifica restrição semanal frequência grupo de prato
             foreach($row['PRIMEIRO_ULTIMO_DIA_SEMANA'] as $dtcard){

                $dataInicial = $dtcard['PRIMEIRO_DIA'];
                $dataFinal = $dtcard['ULTIMO_DIA'];
                $nrSemana = $dtcard['NRSEMANA'];
              
                $dadosRestricaoSemanal = $this->retornaDadosRestricaoFreqGrupoPrato($row,$dataInicial,$dataFinal,$nrSemana);
         
                if(!empty($dadosRestricaoSemanal)){
                 foreach($dadosRestricaoSemanal as $restricaoSemanal){
                    
                     if($restricaoSemanal['QTDE_GRUPO'] > $restricaoSemanal['NRMAXFREQGRPRATSEM']){
                         
                         array_push($restricaoAtivaSemanal, $restricaoSemanal );
                     }
                }
                
            }
        }
   
        $result = array(
            
            'restricaoFreqGrupoPratoMaxSemana' => $restricaoAtivaSemanal,
            'restricaoMensalFreqGrupoPratoMax' => $restricaoAtivaMensalMax,
            'restricaoMensalFreqGrupoPratoMin' => $restricaoAtivaMensalMin
            
        );
        
        return $result;
    }
    
    
    
    public function retornaRestricaoFreqPratoMesSemana($row){
        
             $restricaoAtivaMensalMax = [];
             $restricaoAtivaMensalMin = [];
             $restricaoAtivaSemanal = [];
             
             $dataInicial = $row['DATA_INICIAL'];
             $dataFinal = $row['DATA_FINAL'];
             
             //Verifica restrição mensal frequência dos produtos
             $dadosRestricaoMensal = $this->retornaDadosRestricaoFreqPrato($row,$dataInicial,$dataFinal);
             
             if(!empty($dadosRestricaoMensal)){
                 foreach($dadosRestricaoMensal as $restricaoMensal){
                     
                     if($restricaoMensal['QTDE_PLANEJADO'] > $restricaoMensal['NRMAXFREQPRATMES']){
                         
                         array_push($restricaoAtivaMensalMax, $restricaoMensal );
                     }
                     
                     if($restricaoMensal['QTDE_PLANEJADO'] < $restricaoMensal['NRMINFREQPRATMES']){
                         
                         array_push($restricaoAtivaMensalMin, $restricaoMensal );
                     }
                }
                
             }
         
             //Verifica restrição semanal frequência dos produtos
             foreach($row['PRIMEIRO_ULTIMO_DIA_SEMANA'] as $dtcard){

                $dataInicial = $dtcard['PRIMEIRO_DIA'];
                $dataFinal = $dtcard['ULTIMO_DIA'];
                $nrSemana =  $dtcard['NRSEMANA'];
                 
                $dadosRestricaoSemanal = $this->retornaDadosRestricaoFreqPrato($row,$dataInicial,$dataFinal,$nrSemana);
                
                if(!empty($dadosRestricaoSemanal)){
                 foreach($dadosRestricaoSemanal as $restricaoSemanal){
                    
                     if($restricaoSemanal['QTDE_PLANEJADO'] > $restricaoSemanal['NRMAXFREQPRATSEM']){
                         
                         array_push($restricaoAtivaSemanal, $restricaoSemanal );
                     }
                }
            }
        }
        
        $result = array(
            
            'restricaoFreqPratoMaxSemana' => $restricaoAtivaSemanal,
            'restricaoMensalFreqPratoMax' => $restricaoAtivaMensalMax,
            'restricaoMensalFreqPratoMin' => $restricaoAtivaMensalMin
            
        );
        
        return $result;
        
    }
    
    public function verificaParamRestricoesContratuaisCard($row){
        
        $query = "SELECT IDRESTRCARD FROM PARAMPLANEJA
                    WHERE CDFILIAL = :CDFILIAL";
        
        $param = array(
            
            'CDFILIAL' => $row['CDFILIAL']
            
        );
        
        $idrestrcard = $this->entityManager->getConnection()->fetchAll($query,$param);
        return $idrestrcard[0]['IDRESTRCARD'];
        
    }
    
    public function retornaDadosRestricaoFreqProdGrupoPrato($row,$dataInicial,$dataFinal,$nrSemana = null){
        
        $query = "SELECT DISTINCT NRGRUPOPRATO, CDPRODUTO,NRMINPRODGRPRAT, NRMAXPRODGRPRAT,NRMAXPRODGRPRATSEM, QTDE_PROD,:NRSEMANA AS NRSEMANA                       
                FROM
                (SELECT  NVL(FREQ.NMGRUPOPRATO,'SEM GRUPO') AS NMGRUPOPRATO,FREQ.NRGRUPOPRATO, FREQ.CDPRODUTO, FREQ.NMPRODUTO, FREQ.DTVIGENCIA,
                        FREQ.CDFILAUXPRAT, FREQ.CDPRATO, FREQ.NMPRATO, NVL(FREQ.NRMINPRODGRPRAT,0) AS NRMINPRODGRPRAT, NVL(NRMAXPRODGRPRAT,0) AS NRMAXPRODGRPRAT,NVL(NRMAXPRODGRPRATSEM,0) AS NRMAXPRODGRPRATSEM,
                        CASE WHEN FREQ.CDPRATO IS NULL THEN 0 ELSE COUNT(FREQ.CDPRODUTO) OVER (PARTITION BY FREQ.CDFILIAL, FREQ.CDSERVFILI, FREQ.NRAGRUPGRPRAT, FREQ.NRGRUPOPRATOSERV, FREQ.CDPRODUTO) END AS QTDE_PROD
                  FROM(SELECT CASE WHEN FR.CDFILIAL IS NULL THEN CARD.CDFILIAL ELSE FR.CDFILIAL END AS CDFILIAL, 
                              CASE WHEN FR.CDSERVFILI IS NULL THEN CARD.CDSERVFILI ELSE FR.CDSERVFILI END AS CDSERVFILI, 
                              CASE WHEN FR.NRAGRUPGRPRAT IS NULL THEN CARD.NRAGRUPGRPRAT ELSE FR.NRAGRUPGRPRAT END AS NRAGRUPGRPRAT,
                              CASE WHEN FR.NRGRUPOPRATOSERV IS NULL THEN CARD.NRGRUPOPRATOSERV ELSE FR.NRGRUPOPRATOSERV END AS NRGRUPOPRATOSERV, 
                              CASE WHEN FR.NRGRUPOPRATO IS NULL THEN CARD.NRGRUPOPRATO ELSE FR.NRGRUPOPRATO END AS NRGRUPOPRATO,
                              CASE WHEN FR.NMGRUPOPRATO IS NULL THEN CARD.NMGRUPOPRATO ELSE FR.NMGRUPOPRATO END AS NMGRUPOPRATO, 
                              CASE WHEN FR.CDPRODUTO IS NULL THEN CARD.CDPRODUTO ELSE CASE WHEN FR.CDNVPRODUTO < CARD.CDNVPRODUTO THEN CARD.CDPRODUTO ELSE FR.CDPRODUTO END END AS CDPRODUTO,
                              CASE WHEN FR.CDPRODUTO IS NULL THEN CARD.NMPRODUTO ELSE CASE WHEN FR.CDNVPRODUTO < CARD.CDNVPRODUTO THEN CARD.NMPRODUTO ELSE FR.NMPRODUTO END END AS NMPRODUTO, 
                              CARD.CDFILAUXPRAT, CARD.CDPRATO, CARD.NMPRATO, DTVIGENCIA, NRMINPRODGRPRAT, NRMAXPRODGRPRAT,NRMAXPRODGRPRATSEM
                       FROM
                       (SELECT C.CDFILIAL, C.CDSERVFILI, C.DTCARDAPIO, CP.CDTIPOPRAT, TP.NMTIPOPRAT, CNF.DSCOMPLTPMES AS COMPLEMENTO, 
                               CP.NRSEQPRATCAR, CP.CDFILAUXPRAT, CP.CDPRATO, P.NMPRATO, PR.CDPRODUTO, PR.CDARVPROD, PR.NMPRODUTO, PR.SGUNIDADE, PR.CDNVPRODUTO,
                               G.NRGRUPOPRATOSERV, G.NRGRUPOPRATO, G.CDGRUPOPRATO, G.NMGRUPOPRATO, G.NRAGRUPGRPRAT
                               FROM CARDADIA C INNER JOIN COMPCARD CP ON C.CDFILIAL = CP.CDFILIAL AND C.NRCARDAPIO = CP.NRCARDAPIO AND C.DTCARDAPIO BETWEEN TO_DATE(:DTINICIAL, 'DD/MM/YYYY') AND TO_DATE(:DTFINAL, 'DD/MM/YYYY')
                                               LEFT JOIN (SELECT G.CDFILIAL, GS.CDSERVFILI, G.NRGRUPOPRATO,G.NMGRUPOPRATO, G.CDGRUPOPRATO, GS.NRGRUPOPRATOSERV, GS.NRAGRUPGRPRAT, 
                                                                 IT.CDFILAUXPRAT, IT.CDPRATO
                                                            FROM ITGRUPOPRATOSERV IT 
                                                              INNER JOIN GRUPOPRATOSERV GS ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV 
                                                              INNER JOIN GRUPOPRATO G ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO) G 
                                               ON G.CDFILIAL = C.CDFILIAL AND G.CDSERVFILI = C.CDSERVFILI AND G.CDFILAUXPRAT = CP.CDFILAUXPRAT AND G.CDPRATO = CP.CDPRATO
                                               INNER JOIN PRATO P ON P.CDFILAUXPRAT = CP.CDFILAUXPRAT AND P.CDPRATO = CP.CDPRATO
                                               INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = CP.CDTIPOPRAT
                                               INNER JOIN CNFCAMES CNF ON CNF.CDFILIAL = C.CDFILIAL AND CNF.CDSERVFILI = C.CDSERVFILI AND 
                                                                         CNF.DTREFECNFMES = :P_DTMESCARD AND CNF.NRSEQCNFMES = CP.NRSEQPRATCAR,
                                    TABLE(RECEITA_UTIL(CP.CDFILIAL, C.CDSERVFILI, CP.CDPRATO, CP.CDFILAUXPRAT, :P_CDPRODINI, :P_CDPRODFIM ||'Z', NULL,0,0,'S')) R, PRODUTO PR
                         WHERE R.CDPRATOPAI IS NULL AND R.CDPRODUTO = PR.CDPRODUTO 
                           ) CARD
                        INNER JOIN (SELECT FR.CDFILIAL, FR.CDSERVFILI, V.DTVIGENCIA, V.DESCRICAO, IT.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, P.SGUNIDADE, P.CDNVPRODUTO,
                                                GS.NRGRUPOPRATO, G.NMGRUPOPRATO, G.CDGRUPOPRATO, IT.NRGRUPOPRATOSERV, GS.NRAGRUPGRPRAT, IT.NRMINPRODGRPRAT, IT.NRMAXPRODGRPRAT,IT.NRMAXPRODGRPRATSEM
                                          FROM FREQPRODGRPRAT FR INNER JOIN ITFREQPRODGRPRAT IT ON  FR.NRFREQPRODGRPRAT = IT.NRFREQPRODGRPRAT
                                                                 INNER JOIN GRUPOPRATOSERV GS ON  IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV
                                                                 INNER JOIN GRUPOPRATO G ON G.CDFILIAL = FR.CDFILIAL AND G.NRGRUPOPRATO = GS.NRGRUPOPRATO
                                                                 INNER JOIN VIGFREQPRODGRPRAT V ON V.NRVIGENCIA = FR.NRVIGENCIA AND V.IDTIPORESTRICAO = :P_IDTIPORESTRICAO
                                                                        AND V.DTVIGENCIA = SUBSTR((SELECT MAX(CASE WHEN VV.DTVIGENCIAFIM IS NULL THEN 0 ELSE 1 END || DTVIGENCIA)
                                                                                             FROM VIGFREQPRODGRPRAT VV
                                                                                            WHERE (V.CDFILIAL = VV.CDFILIAL) AND (VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO) AND
                                                                                                  ((VV.DTVIGENCIA <= :P_DTMESCARD AND VV.DTVIGENCIAFIM IS NULL) OR 
                                                                                                  (:P_DTMESCARD BETWEEN VV.DTVIGENCIA AND VV.DTVIGENCIAFIM))
                                                                                           ),2)
                                                                 INNER JOIN PRODUTO P ON P.CDPRODUTO = IT.CDPRODUTO 
                                        WHERE G.CDFILIAL = :CDFILIAL AND GS.CDSERVFILI = :CDSERVFILI
                                        )FR ON FR.CDFILIAL = CARD.CDFILIAL AND FR.CDSERVFILI = CARD.CDSERVFILI AND 
                                               FR.NRGRUPOPRATOSERV = CARD.NRGRUPOPRATOSERV AND CARD.CDARVPROD LIKE FR.CDARVPROD||'%'
                      )FREQ
                     LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = FREQ.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO = :P_IDTIPORESTRICAO 
                     INNER JOIN FILIAL F ON  F.CDFILIAL = FREQ.CDFILIAL
                     INNER JOIN SERVICO S ON  S.CDFILIAL = FREQ.CDFILIAL AND S.CDSERVFILI = FREQ.CDSERVFILI 
                     WHERE FREQ.CDPRATO IS NOT  NULL) F
                WHERE (QTDE_PROD = 0) OR (((QTDE_PROD > NRMAXPRODGRPRAT OR QTDE_PROD < NRMINPRODGRPRAT OR QTDE_PROD > NRMAXPRODGRPRATSEM))) AND F.DTVIGENCIA IS NOT NULL 
                GROUP BY  NRGRUPOPRATO,NMGRUPOPRATO, CDPRODUTO, NMPRODUTO, CDFILAUXPRAT, CDPRATO, NMPRATO, NRMINPRODGRPRAT, NRMAXPRODGRPRAT, NRMAXPRODGRPRATSEM, QTDE_PROD";
                      
        $param = array(
            
            'CDFILIAL' => $row['CDFILIAL'],
            'CDSERVFILI' => $row['CDSERVFILI'],
            'DTINICIAL' => $dataInicial,
            'DTFINAL' => $dataFinal,
            'P_IDTIPORESTRICAO' => $row['IDTIPORESTRICAO'],
            'P_DTMESCARD' => $row['DATA_INICIAL'],
            'P_CDPRODINI' =>'0000000000',
            'P_CDPRODFIM' => '9999999999',
            'NRSEMANA' => $nrSemana
            
        );

        $dadosRestricao = $this->entityManager->getConnection()->fetchAll($query, $param);
        return !empty($dadosRestricao) ? $dadosRestricao : array() ;
        
    }
    
    public function retornaDadosRestricaoFreqPrato($row,$dataInicial,$dataFinal,$nrSemana = null){
        
        $query ="SELECT DISTINCT 
                           FREQ.CDPRATO,
                           FREQ.NRMINFREQPRATMES, 
                           FREQ.NRMAXFREQPRATMES, 
                           FREQ.NRMAXFREQPRATSEM,
                           QTDE_PLANEJADO,
                           :NRSEMANA AS NRSEMANA,
                           FREQ.CDFILAUXPRAT
                    FROM
                    (SELECT CASE WHEN FR.CDFILIAL IS NULL THEN CARD.CDFILIAL ELSE FR.CDFILIAL END AS CDFILIAL, 
                            CASE WHEN FR.CDSERVFILI IS NULL THEN CARD.CDSERVFILI ELSE FR.CDSERVFILI END AS CDSERVFILI,  
                            CASE WHEN FR.CDFILAUXPRAT IS NULL THEN CARD.CDFILAUXPRAT ELSE FR.CDFILAUXPRAT END AS CDFILAUXPRAT, 
                            CASE WHEN FR.CDPRATO IS NULL THEN CARD.CDPRATO ELSE FR.CDPRATO END AS CDPRATO, DTVIGENCIA,  NRMINFREQPRATMES, NRMAXFREQPRATMES,NRMAXFREQPRATSEM,
                            COUNT(CARD.CDPRATO) 
                             OVER (PARTITION BY CARD.CDFILIAL, CARD.CDSERVFILI, FR.DTVIGENCIA,CARD.CDPRATO) AS QTDE_PLANEJADO
                    FROM
                    (SELECT C.CDFILIAL, C.CDSERVFILI, CP.CDFILAUXPRAT, CP.CDPRATO
                       FROM CARDADIA C INNER JOIN COMPCARD CP ON C.CDFILIAL = CP.CDFILIAL AND C.NRCARDAPIO = CP.NRCARDAPIO AND C.DTCARDAPIO BETWEEN TO_DATE(:DTINICIAL, 'DD/MM/YYYY') AND TO_DATE(:DTFINAL, 'DD/MM/YYYY')                   
                                       INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = CP.CDTIPOPRAT
                                       INNER JOIN CNFCAMES CNF ON CNF.CDFILIAL = C.CDFILIAL AND CNF.CDSERVFILI = C.CDSERVFILI AND 
                                                                 CNF.DTREFECNFMES = :P_DTMESCARD AND CNF.NRSEQCNFMES = CP.NRSEQPRATCAR
                    ) CARD
                    INNER JOIN 
                    (SELECT FR.CDFILIAL, FR.CDSERVFILI, V.DTVIGENCIA, IT.CDFILAUXPRAT, IT.CDPRATO, IT.NRMINFREQPRATMES, IT.NRMAXFREQPRATMES, IT.NRMAXFREQPRATSEM
                      FROM FREQPRATMES FR INNER JOIN ITFREQPRATMES IT ON  FR.NRFREQPRATMES = IT.NRFREQPRATMES                        
                                            INNER JOIN VIGFREQPRATMES V ON V.NRVIGENCIA = FR.NRVIGENCIA AND V.IDTIPORESTRICAO = :P_IDTIPORESTRICAO 
                                                   AND V.DTVIGENCIA = SUBSTR((SELECT MAX(CASE WHEN VV.DTVIGENCIAFIM IS NULL THEN 0 ELSE 1 END || DTVIGENCIA)
                                                                                                 FROM VIGFREQPRATMES VV
                                                                                                WHERE (V.CDFILIAL = VV.CDFILIAL) AND (VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO) AND
                                                                                                      ((VV.DTVIGENCIA <= :P_DTMESCARD AND VV.DTVIGENCIAFIM IS NULL) OR 
                                                                                                      (:P_DTMESCARD BETWEEN VV.DTVIGENCIA AND VV.DTVIGENCIAFIM))
                                                                                               ),2)
                            
                    )FR ON FR.CDFILIAL = CARD.CDFILIAL AND FR.CDSERVFILI = CARD.CDSERVFILI AND FR.CDFILAUXPRAT = CARD.CDFILAUXPRAT AND FR.CDPRATO = CARD.CDPRATO)FREQ
                    INNER JOIN PRATO P ON  P.CDFILAUXPRAT = FREQ.CDFILAUXPRAT AND P.CDPRATO = FREQ.CDPRATO
                    WHERE ((QTDE_PLANEJADO > FREQ.NRMAXFREQPRATMES OR QTDE_PLANEJADO < FREQ.NRMINFREQPRATMES OR QTDE_PLANEJADO > FREQ.NRMAXFREQPRATSEM))AND FREQ.CDFILIAL = :CDFILIAL AND FREQ.CDSERVFILI = :CDSERVFILI
                    GROUP BY  FREQ.CDPRATO, P.NMPRATO,FREQ.NRMINFREQPRATMES, FREQ.NRMAXFREQPRATMES,
                            QTDE_PLANEJADO,FREQ.NRMAXFREQPRATSEM, FREQ.CDFILAUXPRAT";
                      
        $param = array(
            
            'CDFILIAL' => $row['CDFILIAL'],
            'CDSERVFILI' => $row['CDSERVFILI'],
            'DTINICIAL' => $dataInicial,
            'DTFINAL' => $dataFinal,
            'P_IDTIPORESTRICAO' => $row['IDTIPORESTRICAO'],
            'P_DTMESCARD' => $row['DATA_INICIAL'],
            'NRSEMANA' => $nrSemana
            
        );

        $dadosRestricao = $this->entityManager->getConnection()->fetchAll($query, $param);
        return !empty($dadosRestricao) ? $dadosRestricao : array() ;
        
    }
    
    public function retornaDadosRestricaoFreqGrupoPrato($row,$dataInicial,$dataFinal,$nrSemana = null){
        
        $query ="SELECT DISTINCT NRGRUPOPRATO,  NRMINFREQGRPRATMES, NRMAXFREQGRPRATMES,NRMAXFREQGRPRATSEM, QTDE_GRUPO,:NRSEMANA AS NRSEMANA
                FROM
                (SELECT FREQ.NRGRUPOPRATO, FREQ.CDGRUPOPRATO, 
                        NVL(FREQ.NMGRUPOPRATO,'SEM GRUPO') AS NMGRUPOPRATO, FREQ.CDFILAUXPRAT,NVL(FREQ.NRMINFREQGRPRATMES,0) AS NRMINFREQGRPRATMES, NVL(NRMAXFREQGRPRATMES,0) AS NRMAXFREQGRPRATMES, 
                        NVL(FREQ.NRMAXFREQGRPRATSEM,0) AS NRMAXFREQGRPRATSEM,
                        COUNT(FREQ.CDPRATO) OVER (PARTITION BY FREQ.CDFILIAL, FREQ.CDSERVFILI, FREQ.NRAGRUPGRPRAT, FREQ.NRGRUPOPRATOSERV) AS QTDE_GRUPO
                  FROM(SELECT CASE WHEN FR.CDFILIAL IS NULL THEN CARD.CDFILIAL ELSE FR.CDFILIAL END AS CDFILIAL, CASE WHEN FR.CDSERVFILI IS NULL THEN CARD.CDSERVFILI ELSE FR.CDSERVFILI END AS CDSERVFILI, CARD.DTCARDAPIO, 
                              CASE WHEN FR.NRAGRUPGRPRAT IS NULL THEN CARD.NRAGRUPGRPRAT ELSE FR.NRAGRUPGRPRAT END AS NRAGRUPGRPRAT,
                              CASE WHEN FR.NRGRUPOPRATOSERV IS NULL THEN CARD.NRGRUPOPRATOSERV ELSE FR.NRGRUPOPRATOSERV END AS NRGRUPOPRATOSERV,
                              CASE WHEN FR.NRGRUPOPRATO IS NULL THEN CARD.NRGRUPOPRATO ELSE FR.NRGRUPOPRATO END AS NRGRUPOPRATO, 
                              CASE WHEN FR.CDGRUPOPRATO IS NULL THEN CARD.CDGRUPOPRATO ELSE FR.CDGRUPOPRATO END AS CDGRUPOPRATO, 
                              CASE WHEN FR.NMGRUPOPRATO IS NULL THEN CARD.NMGRUPOPRATO ELSE FR.NMGRUPOPRATO END AS NMGRUPOPRATO,  
                              CARD.CDFILAUXPRAT, CARD.CDPRATO, NRMINFREQGRPRATMES, NRMAXFREQGRPRATMES,NRMAXFREQGRPRATSEM,DTVIGENCIA
                       FROM
                       (SELECT C.CDFILIAL, C.CDSERVFILI, C.DTCARDAPIO, CP.CDFILAUXPRAT, CP.CDPRATO,G.NRGRUPOPRATOSERV, G.NRGRUPOPRATO, G.CDGRUPOPRATO, G.NMGRUPOPRATO, G.NRAGRUPGRPRAT
                               FROM CARDADIA C INNER JOIN COMPCARD CP ON C.CDFILIAL = CP.CDFILIAL AND C.NRCARDAPIO = CP.NRCARDAPIO AND C.DTCARDAPIO BETWEEN TO_DATE(:DTINICIAL, 'DD/MM/YYYY') AND TO_DATE(:DTFINAL, 'DD/MM/YYYY')
                                               LEFT JOIN (SELECT G.CDFILIAL, GS.CDSERVFILI, G.NRGRUPOPRATO,G.NMGRUPOPRATO, G.CDGRUPOPRATO, GS.NRGRUPOPRATOSERV, GS.NRAGRUPGRPRAT, 
                                                                 IT.CDFILAUXPRAT, IT.CDPRATO
                                                            FROM ITGRUPOPRATOSERV IT 
                                                              INNER JOIN GRUPOPRATOSERV GS ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV 
                                                              INNER JOIN GRUPOPRATO G ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO) G 
                                               ON G.CDFILIAL = C.CDFILIAL AND G.CDSERVFILI = C.CDSERVFILI AND G.CDFILAUXPRAT = CP.CDFILAUXPRAT AND G.CDPRATO = CP.CDPRATO
                                               INNER JOIN PRATO P ON P.CDFILAUXPRAT = CP.CDFILAUXPRAT AND P.CDPRATO = CP.CDPRATO
                                               INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = CP.CDTIPOPRAT
                                               INNER JOIN CNFCAMES CNF ON CNF.CDFILIAL = C.CDFILIAL AND CNF.CDSERVFILI = C.CDSERVFILI AND 
                                                                         CNF.DTREFECNFMES = :P_DTMESCARD AND CNF.NRSEQCNFMES = CP.NRSEQPRATCAR
                             WHERE C.CDFILIAL = :CDFILIAL
                                AND C.CDSERVFILI = :CDSERVFILI
                           ) CARD
                        INNER JOIN (SELECT FR.CDFILIAL, FR.CDSERVFILI, V.DTVIGENCIA,  GS.NRGRUPOPRATO, G.NMGRUPOPRATO, G.CDGRUPOPRATO,
                                               IT.NRGRUPOPRATOSERV, GS.NRAGRUPGRPRAT, IT.NRMINFREQGRPRATMES, IT.NRMAXFREQGRPRATMES, IT.NRMAXFREQGRPRATSEM
                                          FROM FREQGRPRATMES FR INNER JOIN ITFREQGRPRATMES IT ON  FR.NRFREQGRPRATMES = IT.NRFREQGRPRATMES
                                                                INNER JOIN GRUPOPRATOSERV GS ON  IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV
                                                                INNER JOIN GRUPOPRATO G ON  G.CDFILIAL = FR.CDFILIAL AND G.NRGRUPOPRATO = GS.NRGRUPOPRATO
                                                                INNER JOIN VIGFREQGRPRATMES V ON V.NRVIGENCIA = FR.NRVIGENCIA AND V.IDTIPORESTRICAO = :P_IDTIPORESTRICAO
                                                                       AND V.DTVIGENCIA = SUBSTR((SELECT MAX(CASE WHEN VV.DTVIGENCIAFIM IS NULL THEN 0 ELSE 1 END || DTVIGENCIA)
                                                                                             FROM VIGFREQGRPRATMES VV
                                                                                            WHERE (V.CDFILIAL = VV.CDFILIAL) AND (VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO) AND
                                                                                                  ((VV.DTVIGENCIA <= :P_DTMESCARD AND VV.DTVIGENCIAFIM IS NULL) OR 
                                                                                                  (:P_DTMESCARD BETWEEN VV.DTVIGENCIA AND VV.DTVIGENCIAFIM))
                                                                                           ),2)
                                        )FR ON FR.CDFILIAL = CARD.CDFILIAL AND FR.CDSERVFILI = CARD.CDSERVFILI AND FR.NRGRUPOPRATOSERV = CARD.NRGRUPOPRATOSERV
                      )FREQ
                     LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = FREQ.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO = :P_IDTIPORESTRICAO
                     INNER JOIN FILIAL F ON  F.CDFILIAL = FREQ.CDFILIAL
                     INNER JOIN SERVICO S ON  S.CDFILIAL = FREQ.CDFILIAL AND S.CDSERVFILI = FREQ.CDSERVFILI
                     WHERE FREQ.CDFILIAL = :CDFILIAL AND FREQ.CDSERVFILI = :CDSERVFILI AND FREQ.DTVIGENCIA IS NOT NULL) F
                WHERE (QTDE_GRUPO = 0) OR (((QTDE_GRUPO > NRMAXFREQGRPRATMES OR QTDE_GRUPO < NRMINFREQGRPRATMES OR QTDE_GRUPO > NRMAXFREQGRPRATSEM )))     
                GROUP BY  
                       NRGRUPOPRATO, CDGRUPOPRATO, NMGRUPOPRATO, 
                       CDFILAUXPRAT, NRMINFREQGRPRATMES, NRMAXFREQGRPRATMES,NRMAXFREQGRPRATSEM, QTDE_GRUPO";
                      
        $param = array(
            
            'CDFILIAL' => $row['CDFILIAL'],
            'CDSERVFILI' => $row['CDSERVFILI'],
            'DTINICIAL' => $dataInicial,
            'DTFINAL' => $dataFinal,
            'P_IDTIPORESTRICAO' => $row['IDTIPORESTRICAO'],
            'P_DTMESCARD' => $row['DATA_INICIAL'],
            'NRSEMANA' => $nrSemana
            
        );
     
        $dadosRestricao = $this->entityManager->getConnection()->fetchAll($query, $param);
        return !empty($dadosRestricao) ? $dadosRestricao : array() ;
        
    }
    
    public function retornaDadosRestricaoProdGrupoPrato($row){
        $query = "SELECT DISTINCT  REST.NRGRUPOPRATO,  P.CDPRODUTO, REST.QTDE_PROD_PLAN                      
                FROM(
                     SELECT C.CDFILIAL, C.CDSERVFILI,G.NRGRUPOPRATO,G.NMGRUPOPRATO, R.CDPRODUTO,CP.CDFILAUXPRAT, CP.CDPRATO, P.NMPRATO, 
                            COUNT(R.CDPRODUTO) OVER (PARTITION BY C.CDFILIAL, C.CDSERVFILI,V.DTVIGENCIA, AG.CDAGRUPGRPRAT, GS.NRGRUPOPRATOSERV, R.CDPRODUTO) AS QTDE_PROD_PLAN
                         FROM CARDADIA C INNER JOIN COMPCARD CP ON  C.CDFILIAL = CP.CDFILIAL AND C.NRCARDAPIO = CP.NRCARDAPIO AND C.DTMESCARD = :P_DTMESCARD 
                                        INNER JOIN GRUPOPRATOSERV GS ON  GS.CDSERVFILI = C.CDSERVFILI  
                                        INNER JOIN GRUPOPRATO G ON C.CDFILIAL = G.CDFILIAL  AND G.NRGRUPOPRATO = GS.NRGRUPOPRATO  AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO
                                        INNER JOIN ITGRUPOPRATOSERV IG ON  IG.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV AND 
                                                                          IG.CDFILAUXPRAT = CP.CDFILAUXPRAT AND IG.CDPRATO = CP.CDPRATO
                                        INNER JOIN AGRUPGRPRAT AG ON  AG.NRAGRUPGRPRAT = GS.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO = :P_IDTIPORESTRICAO 
                                        INNER JOIN RESTPRODGR RP ON RP.CDFILIAL = C.CDFILIAL AND RP.CDSERVFILI = C.CDSERVFILI AND RP.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV
                                        INNER JOIN (SELECT NRVIGENCIA, CDFILIAL, DTVIGENCIA, DESCRICAO 
                                                      FROM VIGRESTPRODGR V
                                                     WHERE V.IDTIPORESTRICAO = :P_IDTIPORESTRICAO AND
                                                           DTVIGENCIA = SUBSTR((SELECT MAX(CASE WHEN VV.DTVIGENCIAFIM IS NULL THEN 0 ELSE 1 END || DTVIGENCIA)
                                                                                             FROM VIGRESTPRODGR VV
                                                                                            WHERE (V.CDFILIAL = VV.CDFILIAL) AND (VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO) AND 
                                                                                                  ((VV.DTVIGENCIA <= :P_DTMESCARD AND VV.DTVIGENCIAFIM IS NULL) OR 
                                                                                                  (:P_DTMESCARD BETWEEN VV.DTVIGENCIA AND VV.DTVIGENCIAFIM))
                                                                                           ),2)) V
                                        ON V.NRVIGENCIA = RP.NRVIGENCIA
                                        INNER JOIN PRATO P ON P.CDFILAUXPRAT = CP.CDFILAUXPRAT AND P.CDPRATO = CP.CDPRATO
                                        INNER JOIN CNFCAMES CNF ON CNF.CDFILIAL = C.CDFILIAL AND CNF.CDSERVFILI = C.CDSERVFILI AND 
                                                                   CNF.DTREFECNFMES = :P_DTMESCARD AND CNF.NRSEQCNFMES = CP.NRSEQPRATCAR,
                                        TABLE(RECEITA_UTIL(CP.CDFILIAL, RP.CDSERVFILI, CP.CDPRATO, CP.CDFILAUXPRAT, :P_CDPRODINI, :P_CDPRODFIM ||'Z', NULL,0,0,'S')) R
                                        WHERE R.CDPRATOPAI IS NULL AND R.CDPRODUTO = RP.CDPRODUTO AND C.CDFILIAL = :CDFILIAL AND C.CDSERVFILI = :CDSERVFILI) REST
                INNER JOIN FILIAL F ON  F.CDFILIAL = REST.CDFILIAL
                INNER JOIN PRODUTO P ON P.CDPRODUTO = REST.CDPRODUTO AND P.CDPRODUTO = P.CDPRODESTO
                GROUP BY REST.NRGRUPOPRATO, REST.NMGRUPOPRATO, P.CDPRODUTO, P.NMPRODUTO, REST.CDFILAUXPRAT, REST.CDPRATO, REST.NMPRATO, REST.QTDE_PROD_PLAN";
                      
        $param = array(
            
            'CDFILIAL' => $row['CDFILIAL'],
            'CDSERVFILI' => $row['CDSERVFILI'],
            'P_IDTIPORESTRICAO' => $row['IDTIPORESTRICAO'],
            'P_DTMESCARD' => $row['DATA_INICIAL'],
            'P_CDPRODINI' =>'0000000000',
            'P_CDPRODFIM' => '9999999999'
            
        );

        $dadosRestricao = $this->entityManager->getConnection()->fetchAll($query, $param);
        return !empty($dadosRestricao) ? $dadosRestricao : array() ;
        
    }
    
    public function retornaDadosFreqGrupoPratoOpcao($row, $dataInicial, $dataFinal,$nrSemana = null){
        $query = "SELECT  NRGRUPOPRATO, NRMINFREQOPCAGR, NRMAXFREQOPCAGR,NRMAXFREQOPCAGRSEM, QTDE_GRUPO, :NRSEMANA AS NRSEMANA
                FROM
                (SELECT FREQ.NRGRUPOPRATO, NVL(FREQ.NMGRUPOPRATO,'SEM GRUPO') AS NMGRUPOPRATO, FREQ.CDFILAUXPRAT, NVL(FREQ.NRMINFREQOPCAGR,0) AS NRMINFREQOPCAGR, 
                        NVL(NRMAXFREQOPCAGR,0) AS NRMAXFREQOPCAGR,NVL(FREQ.NRMAXFREQOPCAGRSEM,0) AS NRMAXFREQOPCAGRSEM,
                        COUNT(FREQ.CDPRATO) OVER (PARTITION BY FREQ.CDFILIAL, FREQ.CDSERVFILI, FREQ.NRAGRUPGRPRAT, FREQ.NRGRUPOPRATOSERV) AS QTDE_GRUPO
                  FROM(SELECT CASE WHEN FR.CDFILIAL IS NULL THEN CARD.CDFILIAL ELSE FR.CDFILIAL END AS CDFILIAL, 
                              CASE WHEN FR.CDSERVFILI IS NULL THEN CARD.CDSERVFILI ELSE FR.CDSERVFILI END AS CDSERVFILI,
                              CASE WHEN FR.NRAGRUPGRPRAT IS NULL THEN CARD.NRAGRUPGRPRAT ELSE FR.NRAGRUPGRPRAT END AS NRAGRUPGRPRAT,
                              CASE WHEN FR.NRGRUPOPRATOSERV IS NULL THEN CARD.NRGRUPOPRATOSERV ELSE FR.NRGRUPOPRATOSERV END AS NRGRUPOPRATOSERV,
                              CASE WHEN FR.NRGRUPOPRATO IS NULL THEN CARD.NRGRUPOPRATO ELSE FR.NRGRUPOPRATO END AS NRGRUPOPRATO, 
                              CASE WHEN FR.NMGRUPOPRATO IS NULL THEN CARD.NMGRUPOPRATO ELSE FR.NMGRUPOPRATO END AS NMGRUPOPRATO, 
                              CARD.CDFILAUXPRAT, CARD.CDPRATO, DTVIGENCIA, NRMINFREQOPCAGR, NRMAXFREQOPCAGR,NRMAXFREQOPCAGRSEM
                       FROM
                       (SELECT C.CDFILIAL, C.CDSERVFILI,CP.CDFILAUXPRAT, CP.CDPRATO,  G.NRGRUPOPRATOSERV, G.NRGRUPOPRATO, G.NMGRUPOPRATO, G.NRAGRUPGRPRAT
                               FROM CARDADIA C INNER JOIN COMPCARD CP ON C.CDFILIAL = CP.CDFILIAL AND C.NRCARDAPIO = CP.NRCARDAPIO AND C.DTCARDAPIO BETWEEN TO_DATE(:DTINICIAL, 'DD/MM/YYYY') AND TO_DATE(:DTFINAL, 'DD/MM/YYYY')
                                               LEFT JOIN (SELECT G.CDFILIAL, GS.CDSERVFILI, G.NRGRUPOPRATO,G.NMGRUPOPRATO, G.CDGRUPOPRATO, GS.NRGRUPOPRATOSERV, GS.NRAGRUPGRPRAT, 
                                                                 IT.CDFILAUXPRAT, IT.CDPRATO
                                                            FROM ITGRUPOPRATOSERV IT 
                                                              INNER JOIN GRUPOPRATOSERV GS ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV 
                                                              INNER JOIN GRUPOPRATO G ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO) G 
                                               ON G.CDFILIAL = C.CDFILIAL AND G.CDSERVFILI = C.CDSERVFILI AND G.CDFILAUXPRAT = CP.CDFILAUXPRAT AND G.CDPRATO = CP.CDPRATO
                                               INNER JOIN PRATO P ON P.CDFILAUXPRAT = CP.CDFILAUXPRAT AND P.CDPRATO = CP.CDPRATO
                                               INNER JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = CP.CDTIPOPRAT
                                               INNER JOIN CNFCAMES CNF ON CNF.CDFILIAL = C.CDFILIAL AND CNF.CDSERVFILI = C.CDSERVFILI AND 
                                                                         CNF.DTREFECNFMES = :P_DTMESCARD AND CNF.NRSEQCNFMES = CP.NRSEQPRATCAR AND CNF.IDOPCCNFMES = 'S'
                                WHERE C.CDFILIAL = :CDFILIAL
                                    AND C.CDSERVFILI = :CDSERVFILI
                
                           ) CARD
                        INNER JOIN (SELECT FR.CDFILIAL, FR.CDSERVFILI, V.DTVIGENCIA, V.DESCRICAO, GS.NRGRUPOPRATO, G.NMGRUPOPRATO, G.CDGRUPOPRATO,
                                               IT.NRGRUPOPRATOSERV, GS.NRAGRUPGRPRAT, IT.NRMINFREQOPCAGR, IT.NRMAXFREQOPCAGR,IT.NRMAXFREQOPCAGRSEM
                                          FROM FREQOPCAGR FR INNER JOIN ITFREQOPCAGR IT ON  FR.NRFREQOPCAGR = IT.NRFREQOPCAGR
                                                                INNER JOIN GRUPOPRATOSERV GS ON  IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV
                                                                INNER JOIN GRUPOPRATO G ON  G.CDFILIAL = FR.CDFILIAL AND G.NRGRUPOPRATO = GS.NRGRUPOPRATO
                                                                INNER JOIN VIGFREQOPCAGR V ON V.NRVIGENCIA = FR.NRVIGENCIA AND V.IDTIPORESTRICAO = :P_IDTIPORESTRICAO 
                                                                       AND V.DTVIGENCIA = SUBSTR((SELECT MAX(CASE WHEN VV.DTVIGENCIAFIM IS NULL THEN 0 ELSE 1 END || DTVIGENCIA)
                                                                                             FROM VIGFREQOPCAGR VV
                                                                                            WHERE (V.CDFILIAL = VV.CDFILIAL) AND (VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO) AND
                                                                                                  ((VV.DTVIGENCIA <= :P_DTMESCARD AND VV.DTVIGENCIAFIM IS NULL) OR 
                                                                                                  (:P_DTMESCARD BETWEEN VV.DTVIGENCIA AND VV.DTVIGENCIAFIM))
                                                                                           ),2)
                                        )FR ON FR.CDFILIAL = CARD.CDFILIAL AND FR.CDSERVFILI = CARD.CDSERVFILI AND FR.NRGRUPOPRATOSERV = CARD.NRGRUPOPRATOSERV
                      )FREQ
                     LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = FREQ.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO = :P_IDTIPORESTRICAO
                     WHERE FREQ.CDPRATO IS NOT NULL AND FREQ.DTVIGENCIA IS NOT NULL) F
                WHERE (QTDE_GRUPO = 0) OR (((QTDE_GRUPO > NRMAXFREQOPCAGR OR QTDE_GRUPO < NRMINFREQOPCAGR OR QTDE_GRUPO > NRMAXFREQOPCAGRSEM))) 
                GROUP BY NRGRUPOPRATO, NMGRUPOPRATO,   
                       CDFILAUXPRAT,  NRMINFREQOPCAGR, NRMAXFREQOPCAGR,NRMAXFREQOPCAGRSEM, QTDE_GRUPO";
                      
        $param = array(
            
            'CDFILIAL' => $row['CDFILIAL'],
            'CDSERVFILI' => $row['CDSERVFILI'],
            'DTINICIAL' => $dataInicial,
            'DTFINAL' => $dataFinal,
            'P_IDTIPORESTRICAO' => $row['IDTIPORESTRICAO'],
            'P_DTMESCARD' => $row['DATA_INICIAL'],
            'NRSEMANA' => $nrSemana
            
        );

        $dadosRestricao = $this->entityManager->getConnection()->fetchAll($query, $param);
        return !empty($dadosRestricao) ? $dadosRestricao : array() ;
    }
    
    public function retornaDadosLimitPerCapita($row, $dataInicial, $dataFinal){
        $query = "SELECT DISTINCT  NRGRUPOPRATO, CDPRODUTO, QTMINLIMPERSERGR, QTMAXLIMPERSERGR, QTDE_PLAN                        
            FROM
            (SELECT FREQ.NRGRUPOPRATO, 
                    NVL(FREQ.NMGRUPOPRATO,'SEM GRUPO') AS NMGRUPOPRATO, FREQ.CDPRODUTO,  FREQ.NMPRODUTO, 
                    FREQ.CDFILAUXPRAT, FREQ.CDPRATO, FREQ.NMPRATO, NVL(FREQ.QTMINLIMPERSERGR,0) AS QTMINLIMPERSERGR, NVL(QTMAXLIMPERSERGR,0) AS QTMAXLIMPERSERGR,
                    CASE WHEN FREQ.CDPRATO IS NULL 
                        THEN 0 
                        ELSE ROUND(AVG(FREQ.QTPRODUTO) OVER (PARTITION BY FREQ.CDFILIAL, FREQ.CDSERVFILI, FREQ.NRAGRUPGRPRAT, FREQ.NRGRUPOPRATOSERV, FREQ.CDPRODUTO),5) END AS QTDE_PLAN
              FROM(SELECT CASE WHEN FR.CDFILIAL IS NULL THEN CARD.CDFILIAL ELSE FR.CDFILIAL END AS CDFILIAL, 
                          CASE WHEN FR.CDSERVFILI IS NULL THEN CARD.CDSERVFILI ELSE FR.CDSERVFILI END AS CDSERVFILI, 
                          CASE WHEN FR.NRAGRUPGRPRAT IS NULL THEN CARD.NRAGRUPGRPRAT ELSE FR.NRAGRUPGRPRAT END AS NRAGRUPGRPRAT,
                          CASE WHEN FR.NRGRUPOPRATOSERV IS NULL THEN CARD.NRGRUPOPRATOSERV ELSE FR.NRGRUPOPRATOSERV END AS NRGRUPOPRATOSERV,
                          CASE WHEN FR.NRGRUPOPRATO IS NULL THEN CARD.NRGRUPOPRATO ELSE FR.NRGRUPOPRATO END AS NRGRUPOPRATO, 
                          CASE WHEN FR.NMGRUPOPRATO IS NULL THEN CARD.NMGRUPOPRATO ELSE FR.NMGRUPOPRATO END AS NMGRUPOPRATO, 
                          CASE WHEN FR.CDPRODUTO IS NULL THEN CARD.CDPRODUTO ELSE CASE WHEN FR.CDNVPRODUTO < CARD.CDNVPRODUTO THEN CARD.CDPRODUTO ELSE FR.CDPRODUTO END END AS CDPRODUTO,
                          CASE WHEN FR.CDPRODUTO IS NULL THEN CARD.NMPRODUTO ELSE CASE WHEN FR.CDNVPRODUTO < CARD.CDNVPRODUTO THEN CARD.NMPRODUTO ELSE FR.NMPRODUTO END END AS NMPRODUTO,
                          CARD.QTPRODUTO, CARD.CDFILAUXPRAT, CARD.CDPRATO, CARD.NMPRATO, DTVIGENCIA, QTMINLIMPERSERGR, QTMAXLIMPERSERGR
                   FROM
                   (SELECT C.CDFILIAL, C.CDSERVFILI, CP.CDFILAUXPRAT, CP.CDPRATO, P.NMPRATO, PR.CDPRODUTO,PR.CDARVPROD, PR.NMPRODUTO, PR.CDNVPRODUTO, 
                           ROUND((R.QTPRODUTO*((C.QTCOMESTICAR*CP.VRPECONSCARD)/100)/C.QTCOMESTICAR),5) AS QTPRODUTO,
                           G.NRGRUPOPRATOSERV, G.NRGRUPOPRATO, G.NMGRUPOPRATO, G.NRAGRUPGRPRAT
                           FROM CARDADIA C INNER JOIN COMPCARD CP ON C.CDFILIAL = CP.CDFILIAL AND C.NRCARDAPIO = CP.NRCARDAPIO AND C.DTMESCARD = :P_DTMESCARD
                                           LEFT JOIN (SELECT G.CDFILIAL, GS.CDSERVFILI, G.NRGRUPOPRATO,G.NMGRUPOPRATO, G.CDGRUPOPRATO, GS.NRGRUPOPRATOSERV, GS.NRAGRUPGRPRAT, 
                                                             IT.CDFILAUXPRAT, IT.CDPRATO
                                                        FROM ITGRUPOPRATOSERV IT 
                                                          INNER JOIN GRUPOPRATOSERV GS ON IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV 
                                                          INNER JOIN GRUPOPRATO G ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO AND G.IDTIPORESTRICAO = :P_IDTIPORESTRICAO) G 
                                           ON G.CDFILIAL = C.CDFILIAL AND G.CDSERVFILI = C.CDSERVFILI AND G.CDFILAUXPRAT = CP.CDFILAUXPRAT AND G.CDPRATO = CP.CDPRATO
                                           INNER JOIN PRATO P ON P.CDFILAUXPRAT = CP.CDFILAUXPRAT AND P.CDPRATO = CP.CDPRATO
                                           INNER JOIN CNFCAMES CNF ON CNF.CDFILIAL = C.CDFILIAL AND CNF.CDSERVFILI = C.CDSERVFILI AND 
                                                                     CNF.DTREFECNFMES = :P_DTMESCARD AND CNF.NRSEQCNFMES = CP.NRSEQPRATCAR,
                                TABLE(RECEITA_UTIL(CP.CDFILIAL, C.CDSERVFILI, CP.CDPRATO, CP.CDFILAUXPRAT, :P_CDPRODINI, :P_CDPRODFIM ||'Z', NULL,0,0,'S')) R, PRODUTO PR
                     WHERE R.CDPRATOPAI IS NULL AND R.CDPRODUTO = PR.CDPRODUTO 
                       ) CARD
                    INNER JOIN (SELECT FR.CDFILIAL, FR.CDSERVFILI, V.DTVIGENCIA, V.DESCRICAO, IT.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, P.SGUNIDADE, P.CDNVPRODUTO,
                                            GS.NRGRUPOPRATO, G.NMGRUPOPRATO, G.CDGRUPOPRATO, IT.NRGRUPOPRATOSERV, GS.NRAGRUPGRPRAT, IT.QTMINLIMPERSERGR, IT.QTMAXLIMPERSERGR
                                      FROM LIMPESERGR FR INNER JOIN ITLIMPESERGR IT ON  FR.NRLIMPESERGR = IT.NRLIMPESERGR
                                                             INNER JOIN GRUPOPRATOSERV GS ON  IT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV
                                                             INNER JOIN GRUPOPRATO G ON  G.CDFILIAL = FR.CDFILIAL AND G.NRGRUPOPRATO = GS.NRGRUPOPRATO
                                                             INNER JOIN VIGLIMPESERGR V ON V.NRVIGENCIA = FR.NRVIGENCIA AND V.IDTIPORESTRICAO = :P_IDTIPORESTRICAO
                                                                    AND V.DTVIGENCIA = SUBSTR((SELECT MAX(CASE WHEN VV.DTVIGENCIAFIM IS NULL THEN 0 ELSE 1 END || DTVIGENCIA)
                                                                                         FROM VIGLIMPESERGR VV
                                                                                        WHERE (V.CDFILIAL = VV.CDFILIAL) AND (VV.IDTIPORESTRICAO = :P_IDTIPORESTRICAO) AND
                                                                                              ((VV.DTVIGENCIA <= :P_DTMESCARD AND VV.DTVIGENCIAFIM IS NULL) OR 
                                                                                              (:P_DTMESCARD BETWEEN VV.DTVIGENCIA AND VV.DTVIGENCIAFIM))
                                                                                       ),2)
                                                             INNER JOIN PRODUTO P ON P.CDPRODUTO = IT.CDPRODUTO                        
                                    )FR ON FR.CDFILIAL = CARD.CDFILIAL AND FR.CDSERVFILI = CARD.CDSERVFILI AND 
                                           FR.NRGRUPOPRATOSERV = CARD.NRGRUPOPRATOSERV AND CARD.CDARVPROD LIKE FR.CDARVPROD||'%'
                  )FREQ
                 LEFT JOIN AGRUPGRPRAT AG ON AG.NRAGRUPGRPRAT = FREQ.NRAGRUPGRPRAT AND AG.IDTIPORESTRICAO = :P_IDTIPORESTRICAO
            WHERE FREQ.CDFILIAL = :CDFILIAL AND FREQ.CDSERVFILI = :CDSERVFILI and FREQ.DTVIGENCIA IS NOT NULL) F";
                      
        $param = array(
            
            'CDFILIAL' => $row['CDFILIAL'],
            'CDSERVFILI' => $row['CDSERVFILI'],
            'P_IDTIPORESTRICAO' => $row['IDTIPORESTRICAO'],
            'P_DTMESCARD' => $row['DATA_INICIAL'],
            'P_CDPRODINI' =>'0000000000',
            'P_CDPRODFIM' => '9999999999'
            
        );

        $dadosRestricao = $this->entityManager->getConnection()->fetchAll($query, $param);
        return !empty($dadosRestricao) ? $dadosRestricao : array();
    }
    
    
    
    //Update valida Cardápio
    
    public function updateValidaCardapio($card){
        
      
        $cdfilial = $card['CDFILIAL'];
        $cdservfili = $card['CDSERVFILI'];
        $dtMesCardapio = $card['DATA_INICIAL'];
        $dtvalidacao = date("d/m/Y H:i:s");
        $idvalidado = 'S';
        $cdoperadorValidacao = $this->environment->getCdOperador();
        
        $verificaCardames = $this->verificaCardames($cdfilial,$cdservfili,$dtMesCardapio);

        if(empty($verificaCardames)){
            $this->insertCardames($cdfilial,$cdservfili,$dtMesCardapio);
        }

        
        try{
            $this->connection->beginTransaction();
                
            $sql = "UPDATE CARDAMES 
                    SET
                        IDVALIDADO = :IDVALIDADO,
                        CDOPERVALIDACAO = :CDOPERVALIDACAO,
                        DTVALIDACAO = :DTVALIDACAO
                    WHERE
                        CDFILIAL =:CDFILIAL AND
                        CDSERVFILI =:CDSERVFILI AND
                        DTMESCARDAPIO =:DTMESCARDAPIO";

    		$params = array(
                'CDFILIAL' => $cdfilial,
                'CDSERVFILI' => $cdservfili,
                'CDOPERVALIDACAO' => $cdoperadorValidacao,
                'DTMESCARDAPIO' => $dtMesCardapio,
                'IDVALIDADO' => $idvalidado,
                'DTVALIDACAO' => $dtvalidacao
    		);
    
    		$query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            
            $this->connection->commit();
        }catch(\Exception $e){
            $this->connection->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            
        }
    }
    
    public function verificaCardames($cdfilial,$cdservfili,$dtMesCardapio){
        
        $query = "SELECT * FROM CARDAMES
                WHERE CDFILIAL = :CDFILIAL
                    AND CDSERVFILI = :CDSERVFILI
                    AND DTMESCARDAPIO = :DTMESCARDAPIO";
                    
        $params = array(
            
            'CDFILIAL' => $cdfilial,
            'CDSERVFILI' => $cdservfili,
            'DTMESCARDAPIO' => $dtMesCardapio
        
        );
        
        $result = $this->entityManager->getConnection()->fetchAll($query,$params);
        return $result ? $result[0] : array();
        
        
    }
    
    public function insertCardames($cdfilial,$cdservfili,$dtMesCardapio){
        
        try{
            $this->connection->beginTransaction();
                
            $sql = "INSERT INTO CARDAMES (CDFILIAL, CDSERVFILI, DTMESCARDAPIO, IDVALIDADO) 
                        VALUES (:CDFILIAL, :CDSERVFILI, :DTMESCARDAPIO, :IDVALIDADO)";

    		$params = array(
                'CDFILIAL' => $cdfilial,
                'CDSERVFILI' => $cdservfili,
                'DTMESCARDAPIO' => $dtMesCardapio,
                'IDVALIDADO' => 'N'
    		);
    
    		$query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            
            $this->connection->commit();
        }catch(\Exception $e){
            $this->connection->rollback();
            
        }
        
        
        
    }
    
    private function retornaNrValidacao(){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'NOVO_CODIGO_NT');
        $procedure->addParam(new Param('P_CONTADOR',   Param::PARAM_INPUT,  'VALIDACAOCARDAPIO',        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT,  NULL,      Param::PARAM_TYPE_STR, 20));
        $procedure->addParam(new Param('P_QTDE',       Param::PARAM_INPUT,  1, Param::PARAM_TYPE_STR));
        $resultado = $procedure->execute();
        return intVal($resultado['P_SEQUENCIAL']);
    }
    
    private function insereLogValidacao($nrValidacao, &$cardapio){
        try{
            $this->connection->beginTransaction();
            $nrLogValidacaoCardapio = $this->retornaNrLogValidacaoCardapio($nrValidacao, $cardapio);
            $validacoes = $cardapio['VALIDACOES'];
            if(!empty($validacoes['ComensaisTipoPratoNaoOpcao'])){
                $this->insereLogComensaisTP($nrLogValidacaoCardapio, $validacoes['ComensaisTipoPratoNaoOpcao']['comensaisForaValorBase']);
            }
            if(!empty($validacoes['CustoCardapioForaPadrao'])){
                $this->insereLogCustoForaMeta($nrLogValidacaoCardapio, $validacoes['CustoCardapioForaPadrao']['custoForaMeta']);
            }
            $this->insereLogRestricoes($nrLogValidacaoCardapio, $validacoes['RestricoesContratuais']);
            $this->connection->commit(); 
        }catch(\Exception $e){
            $this->connection->rollback();
            $cardapio['ERRO'] = $e->getMessage();
        }
    }
    
    private function retornaNrLogValidacaoCardapio($nrValidacao, $cardapio){
        $sql = "INSERT INTO LOGVALIDACARD
                (NRLOGVALIDACARD, NRVALIDACAO, CDFILIAL, CDSERVFILI, DTMESCARD, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVALIDACARD.NEXTVAL, :NRVALIDACAO, :CDFILIAL, :CDSERVFILI, TO_DATE(:DTMESCARD, 'DD/MM/YYYY'), :CDOPERADOR, :CDOPERADOR)";
        $params = array(
            'NRVALIDACAO' => $nrValidacao,
            'CDFILIAL' => $cardapio['CDFILIAL'],
            'CDSERVFILI' => $cardapio['CDSERVFILI'],
            'DTMESCARD' => $cardapio['DTMESCARDAPIO'],
            'CDOPERADOR' => $this->environment->getCdOperador()
		);
		
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        
        //Obtém chave do log do cardápio
        $sql = "SELECT NRLOGVALIDACARD
                  FROM LOGVALIDACARD
                 WHERE NRVALIDACAO = :NRVALIDACAO
                   AND CDFILIAL = :CDFILIAL
                   AND CDSERVFILI = :CDSERVFILI
                   AND DTMESCARD = TO_DATE(:DTMESCARD, 'DD/MM/YYYY')";
        
        $params = array(
            'NRVALIDACAO' => $nrValidacao,
            'CDFILIAL' => $cardapio['CDFILIAL'],
            'CDSERVFILI' => $cardapio['CDSERVFILI'],
            'DTMESCARD' => $cardapio['DTMESCARDAPIO']
		);
		
		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        if(count($result) > 0)
            return $result[0]['NRLOGVALIDACARD'];
        else
            return null;
    }
    
    private function insereLogComensaisTP($nrLogValidacaoCardapio, $comensais){
        $sql = "INSERT INTO LOGVDCARDCOMEN
                (NRLOGVDCARDCOMEN, NRLOGVALIDACARD, DTCARDAPIO, CDTIPOPRAT, QTCOMENTPPRAT, QTCOMENDIA, QTDIFCOMEN, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDCOMEN.NEXTVAL, :NRLOGVALIDACARD, TO_DATE(:DTCARDAPIO, 'DD/MM/YYYY'), :CDTIPOPRAT, :QTCOMENTPPRAT, :QTCOMENDIA, :QTDIFCOMEN, :CDOPERADOR, :CDOPERADOR)";
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        foreach($comensais as $comen) {
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'DTCARDAPIO' => $comen['DTCARDAPIO'],
                'CDTIPOPRAT' => $comen['CDTIPOPRAT'],
                'QTCOMENTPPRAT' => $comen['COMENSEST'],
                'QTCOMENDIA' => $comen['COMENSDIA'],
                'QTDIFCOMEN' => $comen['DIFERENCA'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		
            $query->execute($params);
        }
    }   
    
    private function insereLogCustoForaMeta($nrLogValidacaoCardapio, $custo){
        $sql = "UPDATE LOGVALIDACARD
                   SET IDVDCUSTO = 'S',
                       VRCUSTOPERC = :VRCUSTOPERC,
                       VRCUSTOPADRAO = :VRCUSTOPADRAO,
                       VRDIFECUSTO = :VRDIFECUSTO,
                       VRMINIMOCUSTO = :VRMINIMOCUSTO,
                       VRMAXIMOCUSTO = :VRMAXIMOCUSTO
                 WHERE NRLOGVALIDACARD = :NRLOGVALIDACARD";
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $params = array(
            'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
            'VRCUSTOPERC' => $custo['CUSTO'],
            'VRCUSTOPADRAO' => $custo['VRCUSTPADR'],
            'VRDIFECUSTO' => $custo['DIFERENCA'],
            'VRMINIMOCUSTO' => $custo['VRCUSTOMIN'],
            'VRMAXIMOCUSTO' => $custo['VRCUSTOMAX']
		);
		
		
        $query->execute($params);
    }
    
    private function insereLogRestricoes($nrLogValidacaoCardapio, $restricoes){
        if(!empty($restricoes)){
            if(!empty($restricoes['FREQUENCIA_PRATO'])){
                $this->insereRestricaoFreqPratMesMin($nrLogValidacaoCardapio, $restricoes['FREQUENCIA_PRATO']['restricaoMensalFreqPratoMin']);
                $this->insereRestricaoFreqPratMesMax($nrLogValidacaoCardapio, $restricoes['FREQUENCIA_PRATO']['restricaoMensalFreqPratoMax']);
                $this->insereRestricaoFreqPratSemana($nrLogValidacaoCardapio, $restricoes['FREQUENCIA_PRATO']['restricaoFreqPratoMaxSemana']);
            }
            if(!empty($restricoes['FREQUENCIA_GRUPO_PRATO'])){
                $this->insereRestricaoFreqGrupPratMesMin($nrLogValidacaoCardapio, $restricoes['FREQUENCIA_GRUPO_PRATO']['restricaoMensalFreqGrupoPratoMin']);
                $this->insereRestricaoFreqGrupPratMesMax($nrLogValidacaoCardapio, $restricoes['FREQUENCIA_GRUPO_PRATO']['restricaoMensalFreqGrupoPratoMax']);
                $this->insereRestricaoFreqGrupPratSemana($nrLogValidacaoCardapio, $restricoes['FREQUENCIA_GRUPO_PRATO']['restricaoFreqGrupoPratoMaxSemana']);
            }
            if(!empty($restricoes['FREQUENCIA_GRUPO_PRATO_OPCAO'])){
                $this->insereRestricaoFreqGrupPratOpcMesMin($nrLogValidacaoCardapio, $restricoes['FREQUENCIA_GRUPO_PRATO_OPCAO']['restricaoMensalFreqGrupoPratoOpcaoMin']);
                $this->insereRestricaoFreqGrupPratOpcMesMax($nrLogValidacaoCardapio, $restricoes['FREQUENCIA_GRUPO_PRATO_OPCAO']['restricaoMensalFreqGrupoPratoOpcaoMax']);
                $this->insereRestricaoFreqGrupPratOpcSemana($nrLogValidacaoCardapio, $restricoes['FREQUENCIA_GRUPO_PRATO_OPCAO']['restricaoFreqGrupoPratoOpcaoMaxSemana']);
            }
            if(!empty($restricoes['FREQUENCIA_PRODUTO_GRUPO_PRATO'])){
                $this->insereRestricaoFreqProdGrupPratMesMin($nrLogValidacaoCardapio, $restricoes['FREQUENCIA_PRODUTO_GRUPO_PRATO']['restricaoMensalProdFreqGrupoPratoMin']);
                $this->insereRestricaoFreqProdGrupPratMesMax($nrLogValidacaoCardapio, $restricoes['FREQUENCIA_PRODUTO_GRUPO_PRATO']['restricaoMensalProdFreqGrupoPratoMax']);
                $this->insereRestricaoFreqProdGrupPratSemana($nrLogValidacaoCardapio, $restricoes['FREQUENCIA_PRODUTO_GRUPO_PRATO']['restricaoFreqProdGrupoPratoMaxSemana']);    
            }
            if(!empty($restricoes['RESTRICAO_PRODUTO_GRUPO_PRATO'])){
                $this->insereRestricaoBloqProdGrupPrat($nrLogValidacaoCardapio, $restricoes['RESTRICAO_PRODUTO_GRUPO_PRATO']['restricaoProdGrupoPrato']);
            }
            if(!empty($restricoes['LIMITE_PER_CAPITA_POR_SERVICO'])){
                $this->insereRestricaoLimitePercServMin($nrLogValidacaoCardapio, $restricoes['LIMITE_PER_CAPITA_POR_SERVICO']['restricaoLimitPerCapitaMin']);
                $this->insereRestricaoLimitePercServMax($nrLogValidacaoCardapio, $restricoes['LIMITE_PER_CAPITA_POR_SERVICO']['restricaoLimitPerCapitaMax']);
            }
            
        }   
    }
    
    private function insereRestricaoFreqPratMesMin($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, CDFILAUXPRAT, CDPRATO, QTOCORRENCIA, QTMINIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'FREQPRATMESMIN', :CDFILAUXPRAT, :CDPRATO, :QTOCORRENCIA, :QTMINIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'CDFILAUXPRAT' => $restr['CDFILAUXPRAT'],
                'CDPRATO' => $restr['CDPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_PLANEJADO'],
                'QTMINIMO' => $restr['NRMINFREQPRATMES'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    private function insereRestricaoFreqPratMesMax($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, CDFILAUXPRAT, CDPRATO, QTOCORRENCIA, QTMAXIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'FREQPRATMESMAX', :CDFILAUXPRAT, :CDPRATO, :QTOCORRENCIA, :QTMAXIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'CDFILAUXPRAT' => $restr['CDFILAUXPRAT'],
                'CDPRATO' => $restr['CDPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_PLANEJADO'],
                'QTMAXIMO' => $restr['NRMAXFREQPRATMES'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
		
		
    }
    
    private function insereRestricaoFreqPratSemana($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, CDFILAUXPRAT, CDPRATO, QTOCORRENCIA, NRSEMANA, QTMAXIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'FREQPRATSEM', :CDFILAUXPRAT, :CDPRATO, :QTOCORRENCIA, :NRSEMANA, :QTMAXIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'CDFILAUXPRAT' => $restr['CDFILAUXPRAT'],
                'CDPRATO' => $restr['CDPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_PLANEJADO'],
                'NRSEMANA' => $restr['NRSEMANA'],
                'QTMAXIMO' => $restr['NRMAXFREQPRATSEM'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    private function insereRestricaoFreqGrupPratMesMin($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, NRGRUPOPRATO, QTOCORRENCIA, QTMINIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'FREQGRUPPRATMESMIN', :NRGRUPOPRATO, :QTOCORRENCIA, :QTMINIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'NRGRUPOPRATO' => $restr['NRGRUPOPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_GRUPO'],
                'QTMINIMO' => $restr['NRMINFREQGRPRATMES'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    private function insereRestricaoFreqGrupPratMesMax($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, NRGRUPOPRATO, QTOCORRENCIA, QTMAXIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'FREQGRUPPRATMESMAX', :NRGRUPOPRATO, :QTOCORRENCIA, :QTMAXIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'NRGRUPOPRATO' => $restr['NRGRUPOPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_GRUPO'],
                'QTMAXIMO' => $restr['NRMAXFREQGRPRATMES'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
		
    }
    
    private function insereRestricaoFreqGrupPratSemana($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, NRGRUPOPRATO, QTOCORRENCIA, NRSEMANA, QTMAXIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'FREQGRUPPRATSEM', :NRGRUPOPRATO, :QTOCORRENCIA, :NRSEMANA, :QTMAXIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'NRGRUPOPRATO' => $restr['NRGRUPOPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_GRUPO'],
                'NRSEMANA' => $restr['NRSEMANA'],
                'QTMAXIMO' => $restr['NRMAXFREQGRPRATSEM'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    private function insereRestricaoFreqGrupPratOpcMesMin($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, NRGRUPOPRATO, QTOCORRENCIA, QTMINIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'FREQOPGRUPPRATMESMIN', :NRGRUPOPRATO, :QTOCORRENCIA, :QTMINIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'NRGRUPOPRATO' => $restr['NRGRUPOPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_GRUPO'],
                'QTMINIMO' => $restr['NRMINFREQOPCAGR'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    private function insereRestricaoFreqGrupPratOpcMesMax($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, NRGRUPOPRATO, QTOCORRENCIA, QTMAXIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'FREQOPGRUPPRATMESMAX', :NRGRUPOPRATO, :QTOCORRENCIA, :QTMAXIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'NRGRUPOPRATO' => $restr['NRGRUPOPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_GRUPO'],
                'QTMAXIMO' => $restr['NRMAXFREQOPCAGR'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    private function insereRestricaoFreqGrupPratOpcSemana($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, NRGRUPOPRATO, QTOCORRENCIA, NRSEMANA, QTMAXIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'FREQOPGRUPPRATSEM', :NRGRUPOPRATO, :QTOCORRENCIA, :NRSEMANA, :QTMAXIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'NRGRUPOPRATO' => $restr['NRGRUPOPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_GRUPO'],
                'NRSEMANA' => $restr['NRSEMANA'],
                'QTMAXIMO' => $restr['NRMAXFREQOPCAGRSEM'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    private function insereRestricaoFreqProdGrupPratMesMin($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, CDPRODUTO, NRGRUPOPRATO, QTOCORRENCIA, QTMINIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'FREQPRODGRUPPRATMESMIN', :CDPRODUTO, :NRGRUPOPRATO, :QTOCORRENCIA, :QTMINIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'CDPRODUTO' => $restr['CDPRODUTO'],
                'NRGRUPOPRATO' => $restr['NRGRUPOPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_PROD'],
                'QTMINIMO' => $restr['NRMINPRODGRPRAT'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    private function insereRestricaoFreqProdGrupPratMesMax($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, CDPRODUTO, NRGRUPOPRATO, QTOCORRENCIA, QTMAXIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'FREQPRODGRUPPRATMESMAX', :CDPRODUTO, :NRGRUPOPRATO, :QTOCORRENCIA, :QTMAXIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'CDPRODUTO' => $restr['CDPRODUTO'],
                'NRGRUPOPRATO' => $restr['NRGRUPOPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_PROD'],
                'QTMAXIMO' => $restr['NRMAXPRODGRPRAT'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    private function insereRestricaoFreqProdGrupPratSemana($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, CDPRODUTO, NRGRUPOPRATO, QTOCORRENCIA, NRSEMANA, QTMAXIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'FREQPRODGRUPPRATSEM', :CDPRODUTO, :NRGRUPOPRATO, :QTOCORRENCIA, :NRSEMANA, :QTMAXIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'CDPRODUTO' => $restr['CDPRODUTO'],
                'NRGRUPOPRATO' => $restr['NRGRUPOPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_PROD'],
                'NRSEMANA' => $restr['NRSEMANA'],
                'QTMAXIMO' => $restr['NRMAXPRODGRPRATSEM'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    private function insereRestricaoBloqProdGrupPrat($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, CDPRODUTO, NRGRUPOPRATO, QTOCORRENCIA, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'RESTRICAOPRODGRUPPRAT', :CDPRODUTO, :NRGRUPOPRATO, :QTOCORRENCIA, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'CDPRODUTO' => $restr['CDPRODUTO'],
                'NRGRUPOPRATO' => $restr['NRGRUPOPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_PROD_PLAN'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    private function insereRestricaoLimitePercServMin($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, CDPRODUTO, NRGRUPOPRATO, QTOCORRENCIA, QTMINIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'LIMITEPERCSERVMIN', :CDPRODUTO, :NRGRUPOPRATO, :QTOCORRENCIA, :QTMINIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'CDPRODUTO' => $restr['CDPRODUTO'],
                'NRGRUPOPRATO' => $restr['NRGRUPOPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_PLAN'],
                'QTMINIMO' => $restr['QTMINLIMPERSERGR'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    private function insereRestricaoLimitePercServMax($nrLogValidacaoCardapio, $restricao){
        if(empty($restricao)) return;
        $sql = "INSERT INTO LOGVDCARDRESTR
                (NRLOGVDCARDRESTR, NRLOGVALIDACARD, IDRESTRICAO, CDPRODUTO, NRGRUPOPRATO, QTOCORRENCIA, QTMAXIMO, CDOPERINCLUSAO, CDOPERULTATU)
                VALUES
                (SEQ_LOGVDCARDRESTR.NEXTVAL, :NRLOGVALIDACARD, 'LIMITEPERCSERVMAX', :CDPRODUTO, :NRGRUPOPRATO, :QTOCORRENCIA, :QTMAXIMO, :CDOPERADOR, :CDOPERADOR)";
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        foreach($restricao as $restr){
            $params = array(
                'NRLOGVALIDACARD' => $nrLogValidacaoCardapio,
                'CDPRODUTO' => $restr['CDPRODUTO'],
                'NRGRUPOPRATO' => $restr['NRGRUPOPRATO'],
                'QTOCORRENCIA' => $restr['QTDE_PLAN'],
                'QTMAXIMO' => $restr['QTMAXLIMPERSERGR'],
                'CDOPERADOR' => $this->environment->getCdOperador()
    		);
    		
    		$query->execute($params);
        }
    }
    
    //Limpeza das tabelas de validação antigas
    private function apagaLogValidacaoAntiga(){
        try{
            $this->connection->beginTransaction();
            $sql = "DELETE FROM LOGVALIDACARD WHERE DTVALIDACAO <= TRUNC(SYSDATE) - 2";
            $query = $this->entityManager->getConnection()->prepare($sql);
    		$query->execute();
    		$this->connection->commit();
        }catch(\Exception $e){
            $this->connection->rollback();
        }
    }
}